unit MACIP;

interface

uses Windows, KOL;

function GetMACAddress( var IdxCount: Integer ): Int64;
{* Returns MAC address of the Ethernet network card.
   On input, set IdxCount to an index of the card (0 is enouth,
   if Count is not yet known). On output, Count always contains
   number of cards. It seems that if the are several virtual
   network devices in addition to the real network device
   (e.g. installed with Virtual Machine emulators), the first card
   returned actually is a real network unit.
   |<br>
   Though only Ethernet cards are looking for, Count contains
   amount of all the interfaces (loopback, tokenring, sleep, etc.)
   So make a loop from 0 to Count-1 until returned value become
   <> 0 to get the MAC addres og the Ethernet network device. }

implementation

const
  MAX_INTERFACE_NAME_LEN = 256;
  MAXLEN_IFDESCR         = 256;
  MAXLEN_PHYSADDR        = 8;

  MIB_IF_TYPE_ETHERNET   = 6;


type
  MIB_IFROW = record
    wszName: array[0..MAX_INTERFACE_NAME_LEN-1] of WCHAR;
    dwIndex: DWORD;
    dwType : DWORD;
    dwMtu  : DWORD;
    dwSpeed: DWORD;
    dwPhysAddrLen: DWORD;
    bPhysAddr: array[0..MAXLEN_PHYSADDR-1] of BYTE;
    dwAdminStatus: DWORD;
    dwOperStatus : DWORD;
    dwLastChange : DWORD;
    dwInOctets   : DWORD;
    dwInUcastPkts: DWORD;
    dwInNUcastPkts: DWORD;
    dwInDiscards  : DWORD;
    dwInErrors    : DWORD;
    dwInUnknownProtos: DWORD;
    dwOutOctets   : DWORD;
    dwOutUcastPkts: DWORD;
    dwOutNUcastPkts: DWORD;
    dwOutDiscards : DWORD;
    dwOutErrors   : DWORD;
    dwOutQLen     : DWORD;
    dwDescrLen    : DWORD;
    bDescr: array[0..MAXLEN_IFDESCR-1] of BYTE;
  end;

  MIB_IFTABLE = record
    dwNumEntries: DWORD;
    table: array[0..0] of MIB_IFROW;
  end;
  PMIB_IFTABLE = ^MIB_IFTABLE;


type TGetIfTable = function( pIfTable: PMIB_IFTABLE; pdwSize: PDWORD;
     bOrder: BOOL ): DWORD; stdcall;

function GetMACAddress( var IdxCount: Integer ): Int64;
var GetIfTable: TGetIfTable;
    M: THandle;
    Buffer: PMIB_IFTABLE;
    Size: DWORD;
    R: DWORD;
    Addr: array[ 0..7 ] of Byte;
    i, j: DWORD;
    Idx: Integer;
begin
  Result := 0;
  Idx := IdxCount;
  IdxCount := 0;
  M := LoadLibrary( 'Iphlpapi.dll' );
  if M = 0 then Exit;
  TRY
    GetIfTable := GetProcAddress( M, 'GetIfTable' );
    if not Assigned( GetIfTable ) then Exit;

    Size := Sizeof( MIB_IFTABLE );
    Buffer := AllocMem( Size );
    R := GetIfTable( Buffer, @ Size, FALSE );
    if R = ERROR_INSUFFICIENT_BUFFER then
    begin
      FreeMem( Buffer );
      Buffer := AllocMem( Size );
      R := GetIfTable( Buffer, @ Size, FALSE );
    end;

    if (R = NO_ERROR) and (Buffer.dwNumEntries > 0) then
    begin
      IdxCount := Buffer.dwNumEntries;
      i := Idx;
      if i >= IdxCount then
        i := IdxCount-1;
      for i := i to IdxCount-1 do
      begin
        Result := 0;
        if Buffer.table[i].dwType = MIB_IF_TYPE_ETHERNET then
        begin
          Move( Buffer.table[i].bPhysAddr, Addr, 8 );
          for j := 1 to min(8,Buffer.table[i].dwPhysAddrLen) do
            Result := (Result shl 8) or Addr[ j-1 ];
          if Result <> 0 then break;
        end;
      end;
    end;

  FINALLY
    FreeLibrary( M );
  END;
end;

end.
